local Lib = MissionMinder;
Lib.callbacks= Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

local options = MissionMinder.options;
local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local DEBUG = MissionMinder.debug;

MissionMinder.autoclose = {};
local module = MissionMinder.autoclose;

----------------
-- There are a number of ways the CompleteFrame can initiate AutoClose
-- 1. Mission Failed - Close on failure animation finished and after base XP has been awarded
-- 2. Mission Success, No Bonus XP - Close on chest open animation finished
-- 3. Mission Success, Bonus XP - Close after bonus XP has been awarded
----------------


local AUTOCLOSE_FAILED_DELAY = 1.5
local AUTOCLOSE_SUCCESS_LOOT_DELAY = 1.0;
local AUTOCLOSE_SUCCESS_XP_DELAY = 0.7;

local currentMission = nil;
local hasBonusXP = nil;
local hasBeenLooted = nil;

local autocloseTimer = nil;
local function hideCompleteMission()
	autocloseTimer = nil;
	if ( options.ListViewEnabled() ) then
		GarrisonMissionFrame_HideCompleteMissions(true);
	else
		GarrisonMissionCompleteNextButton_OnClick(GarrisonMissionFrame.MissionComplete.NextMissionButton);
	end
end

local function closeCompletionFrame(delay)
	if( autocloseTimer ) then
		autocloseTimer:Cancel();
	end

	DEBUG("Closing in %.1f", delay);
	autocloseTimer = C_Timer.NewTimer(delay, hideCompleteMission);
end

local function disableTimer()
	if( autocloseTimer ) then
		autocloseTimer:Cancel();
		autocloseTimer = nil;
	end
end

local function MissionHasBonusXPReward(missionID)
	local rewards = C_Garrison.GetMissionRewardInfo(missionID);
	for _, reward in pairs(rewards) do
		if(reward.followerXP) then
			return true;
		end
	end

	return false;
end

local function initializeMissionState(completeMissions, index)
	if( options.AutoClose() and GarrisonMissionFrame.MissionComplete:IsShown() ) then
		currentMission = completeMissions[index]
		hasBonusXP = MissionHasBonusXPReward(currentMission.missionID);
		hasBeenLooted = false;
		DEBUG("AutoClose initState hasBonusXP=%s", tostring(hasBonusXP))

		local bonusXPMaxOverride = true
		local self = GarrisonMissionFrame.MissionComplete.Stage.FollowersFrame.Followers;
		for i = 1, #currentMission.followers do
			local _, _, level, quality = C_Garrison.GetFollowerMissionCompleteInfo(currentMission.followers[i]);
			if(hasBonusXP and level == GarrisonMissionFrame.followerMaxLevel and quality == GarrisonMissionFrame.followerMaxQuality) then
				self[i].hasFinishedXPGains = true; -- Follower is already maxxed, so ignore XP gains for them
			else
				self[i].hasFinishedXPGains = false;
				bonusXPMaxOverride = false
			end
			DEBUG("AutoClose initState [%d].hasFinishedXPGains=%s (%d, %d)", i, tostring(self[i].hasFinishedXPGains), level, quality);
		end

		DEBUG("AutoClose initState bonusXPMaxOverride=%s", tostring(bonusXPMaxOverride));
		if ( bonusXPMaxOverride ) then
			hasBonusXP = false;
		end
	end
end

local function removeMissionState()
	currentMission = nil;
	hasBonusXP = nil;
	hasBeenLooted = nil;

	local self = GarrisonMissionFrame.MissionComplete.Stage.FollowersFrame.Followers;
	for i = 1, #self do
		local follower = self[i];
		follower.hasFinishedXPGains = nil;
	end
end


local function autoCloseMissionFailed()
	if ( options.AutoClose() ) then
		if ( currentMission and not currentMission.succeeded ) then
			DEBUG("autoCloseMissionFailed()");
			closeCompletionFrame(AUTOCLOSE_FAILED_DELAY);
		end
	end
end

local function markLooted()
	hasBeenLooted = true;
end

local function closeOnSuccessfulLootNoBonusXP(...)
	if ( options.AutoClose() and not hasBonusXP ) then
		DEBUG("closeOnSuccessfulLootNoBonusXP()");
		closeCompletionFrame(AUTOCLOSE_SUCCESS_LOOT_DELAY);
	end
end


local function checkFollower(followerFrame)
	if ( not hasBeenLooted ) then
		return false;
	end

	if( followerFrame.activeAnims == nil or followerFrame.activeAnims == 0 ) then
		local pendingXPAwards = GarrisonMissionFrame.MissionComplete.pendingXPAwards;
		for k, v in pairs(pendingXPAwards) do
			if ( v.followerID == followerID ) then
				return false; -- has pending rewards
			end
		end

		return true;
	end

	return false;
end

local function isFinishedWithXP()
	local self = GarrisonMissionFrame.MissionComplete.Stage.FollowersFrame.Followers;
	for i = 1, #currentMission.followers do
		local follower = self[i];
		if ( not follower.hasFinishedXPGains ) then
			return false;
		end
	end

	return true;
end

local function closeOnFinishedXP(followerFrame)
	followerFrame.hasFinishedXPGains = checkFollower(followerFrame);

	if ( options.AutoClose() and hasBonusXP and isFinishedWithXP()) then
		DEBUG("closeOnFinishedXP()");
		closeCompletionFrame(AUTOCLOSE_SUCCESS_XP_DELAY); -- Maxed out followers never get to this point as they don't have any XP animation
	end
end

local function initialize()
DEBUG("AutoClose initialized")
--function module.initialize()
	-- Initialize mission state for auto close
	hooksecurefunc("GarrisonMissionComplete_Initialize", initializeMissionState);
	-- Remove misison state on close
	hooksecurefunc("GarrisonMissionFrame_HideCompleteMissions", removeMissionState);

	-- Auto close on successful loot
	hooksecurefunc("GarrisonMissionComplete_OnFollowerXPFinished", closeOnFinishedXP);


	-- Auto close on successful loot (if no bonus XP)
	local bonusRewards = GarrisonMissionFrame.MissionComplete.BonusRewards;
	bonusRewards.ChestModel.OpenAnim:HookScript("OnFinished", closeOnSuccessfulLootNoBonusXP);

	-- Auto close on failure
	GarrisonMissionFrame.MissionComplete.ChanceFrame.ResultAnim:HookScript("OnFinished", autoCloseMissionFailed);

	-- Disable timer on NextMissionButton:OnClick
	local NextMissionButton = GarrisonMissionFrame.MissionComplete.NextMissionButton;
	NextMissionButton:HookScript("OnClick", disableTimer);

	-- Disable timer on MissionComplete:OnHide
	GarrisonMissionFrame.MissionComplete:HookScript("OnHide", disableTimer);
end


local events = Lib.events
events:RegisterEvent("autoclose", "ADDON_LOADED", initialize);
events:RegisterEvent("autoclose", "GARRISON_MISSION_BONUS_ROLL_COMPLETE", markLooted);
